// Squeeze Break Indicator
// Last changed 17 Feb 2010
// Part of Forex Strategy Builder v2.8.3.7+
// Website http://forexsb.com/
// This code or any part of it cannot be used in other applications without a permission.
// Copyright (c) 2010 Denny Imanuel - All rights reserved.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Relative Vigor Index Indicator
    /// </summary>
    public class Squeeze_Break : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public Squeeze_Break(SlotTypes slotType)
        {
            // General properties
            IndicatorName  = "Squeeze Break";
            PossibleSlots  = SlotTypes.OpenFilter | SlotTypes.CloseFilter;
            SeparatedChart = true;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption  = "Logic";
            IndParam.ListParam[0].ItemList = new string[]
            {
                "The Squeeze Break line rises",
                "The Squeeze Break line falls",
                "The Squeeze Break line is higher than zero",
                "The Squeeze Break line is lower than zero",
                "The Squeeze Break line crosses the zero line upward",
                "The Squeeze Break line crosses the zero line downward",
                "The Squeeze Break line changes its direction upward",
                "The Squeeze Break line changes its direction downward",
                "The Squeeze Break line crosses the Signal line upward",
                "The Squeeze Break line crosses the Signal line downward",
                "The Squeeze Break line is higher than the Signal line",
                "The Squeeze Break line is lower than the Signal line"
            };
            IndParam.ListParam[0].Index    = 0;
            IndParam.ListParam[0].Text     = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled  = true;
            IndParam.ListParam[0].ToolTip  = "Logic of application of the indicator.";

            IndParam.ListParam[1].Caption  = "Smoothing method";
            IndParam.ListParam[1].ItemList = Enum.GetNames(typeof(MAMethod));
            IndParam.ListParam[1].Index    = (int)MAMethod.Simple;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
            IndParam.ListParam[1].ToolTip  = "The method of smoothing of central Moving Average.";

            IndParam.ListParam[2].Caption  = "Base price";
            IndParam.ListParam[2].ItemList = Enum.GetNames(typeof(BasePrice));
            IndParam.ListParam[2].Index    = (int)BasePrice.Close;
            IndParam.ListParam[2].Text     = IndParam.ListParam[2].ItemList[IndParam.ListParam[2].Index];
            IndParam.ListParam[2].Enabled  = true;
            IndParam.ListParam[2].ToolTip  = "The price the central Moving Average is based on.";


            // The NumericUpDown parameters
            IndParam.NumParam[0].Caption = "Indicator Period";
            IndParam.NumParam[0].Value   = 20;
            IndParam.NumParam[0].Min     = 1;
            IndParam.NumParam[0].Max     = 200;
            IndParam.NumParam[0].Enabled = true;
            IndParam.NumParam[0].ToolTip = "The Central Moving Average period for Keltner and Bollinger band.";

            IndParam.NumParam[1].Caption = "Signal Period";
            IndParam.NumParam[1].Value   = 10;
            IndParam.NumParam[1].Min     = 1;
            IndParam.NumParam[1].Max     = 50;
            IndParam.NumParam[1].Point   = 0;
            IndParam.NumParam[1].Enabled = true;
            IndParam.NumParam[1].ToolTip = "Average True Range Period.";

            IndParam.NumParam[2].Caption = "Keltner's Multiplier";
            IndParam.NumParam[2].Value   = 2;
            IndParam.NumParam[2].Min     = 1;
            IndParam.NumParam[2].Max     = 10;
            IndParam.NumParam[2].Point   = 1;
            IndParam.NumParam[2].Enabled = true;
            IndParam.NumParam[2].ToolTip = "Average True Range Multiplier.";
			
			IndParam.NumParam[3].Caption = "Bollinger's Multiplier";
            IndParam.NumParam[3].Value   = 2;
            IndParam.NumParam[3].Min     = 1;
            IndParam.NumParam[3].Max     = 10;
            IndParam.NumParam[3].Point   = 1;
            IndParam.NumParam[3].Enabled = true;
            IndParam.NumParam[3].ToolTip = "Determines the width of Bollinger Bands.";

            // The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value";
            IndParam.CheckParam[0].Checked = PrepareUsePrevBarValueCheckBox(slotType);
            IndParam.CheckParam[0].Enabled = true;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
            MAMethod  maMethod = (MAMethod )IndParam.ListParam[1].Index;
            BasePrice price    = (BasePrice)IndParam.ListParam[2].Index;
            int       nMA      = (int)IndParam.NumParam[0].Value;
			int       nSM      = (int)IndParam.NumParam[1].Value;
            double    dKCMpl   = IndParam.NumParam[2].Value;
            double    dBBMpl   = IndParam.NumParam[3].Value;
            int       iPrvs    = IndParam.CheckParam[0].Checked ? 1 : 0;

			double[] adPrice   = Price(price);
			double[] adMA      = MovingAverage(nMA, 0, maMethod, adPrice);
			double[] adATR     = new double[Bars];
			double[] adStdDev  = new double[Bars];
			double[] adBBUp    = new double[Bars];
			double[] adBBDn    = new double[Bars];
			double[] adKCUp    = new double[Bars];
			double[] adKCDn    = new double[Bars];
			double[] adSqzBrk    = new double[Bars];
			double[] adMASignal  = new double[Bars];
			
			double dSum;
			double dDelta;
			
			int iFirstBar = (int)Math.Max(nMA, 14) + iPrvs + 2;
			
			// Calculation of Keltner Channel
			for (int iBar = iFirstBar; iBar < Bars; iBar++)
            {
				dSum = 0;
                for (int i = 0; i < nMA; i++)
                {
                    dDelta = (adPrice[iBar - i] - adMA[iBar]);
                    dSum  += dDelta * dDelta;
                }
                adStdDev[iBar] = Math.Sqrt(dSum / nMA);
			
                adATR[iBar] = Math.Max(Math.Abs(High[iBar] - Close[iBar - 1]), Math.Abs(Close[iBar - 1] - Low[iBar]));
                adATR[iBar] = Math.Max(Math.Abs(High[iBar] - Low[iBar]), adATR[iBar]);
            }

            adATR = MovingAverage(14, 0, maMethod, adATR);
	
            for (int iBar = iFirstBar; iBar < Bars; iBar++)
            {
				adBBUp[iBar] = adMA[iBar] + dBBMpl * adStdDev[iBar];
                adBBDn[iBar] = adMA[iBar] - dBBMpl * adStdDev[iBar];
				
                adKCUp[iBar] = adMA[iBar] + adATR[iBar] * dKCMpl;
                adKCDn[iBar] = adMA[iBar] - adATR[iBar] * dKCMpl;
				
				if (adBBUp[iBar]>=adKCUp[iBar] || adBBDn[iBar]<=adKCDn[iBar]) 
					adSqzBrk[iBar] = Math.Abs(adBBUp[iBar]-adKCUp[iBar])+Math.Abs(adBBDn[iBar]-adKCDn[iBar]);
				if (adBBUp[iBar]<adKCUp[iBar] || adBBDn[iBar]>adKCDn[iBar])
					adSqzBrk[iBar] = -(Math.Abs(adBBUp[iBar]-adKCUp[iBar])+Math.Abs(adBBDn[iBar]-adKCDn[iBar]));	
            }
			
			adMASignal = MovingAverage(nSM, 0, maMethod, adSqzBrk);

            // Saving the components
            Component = new IndicatorComp[4];

            Component[0] = new IndicatorComp();
            Component[0].CompName   = "Squeeze Break";
            Component[0].DataType   = IndComponentType.IndicatorValue;
            Component[0].ChartType  = IndChartType.Histogram;
            Component[0].ChartColor = Color.Green;
            Component[0].FirstBar   = iFirstBar;
            Component[0].Value      = adSqzBrk;

            Component[1] = new IndicatorComp();
            Component[1].CompName   = "Signal line";
            Component[1].DataType   = IndComponentType.IndicatorValue;
            Component[1].ChartType  = IndChartType.Line;
            Component[1].ChartColor = Color.Red;
            Component[1].FirstBar   = iFirstBar;
            Component[1].Value      = adMASignal;

            Component[2] = new IndicatorComp();
            Component[2].ChartType = IndChartType.NoChart;
            Component[2].FirstBar  = iFirstBar;
            Component[2].Value     = new double[Bars];

            Component[3] = new IndicatorComp();
            Component[3].ChartType = IndChartType.NoChart;
            Component[3].FirstBar  = iFirstBar;
            Component[3].Value     = new double[Bars];

            // Sets the Component's type
            if (slotType == SlotTypes.OpenFilter)
            {
                Component[2].DataType = IndComponentType.AllowOpenLong;
                Component[2].CompName = "Is long entry allowed";
                Component[3].DataType = IndComponentType.AllowOpenShort;
                Component[3].CompName = "Is short entry allowed";
            }
            else if (slotType == SlotTypes.CloseFilter)
            {
                Component[2].DataType = IndComponentType.ForceCloseLong;
                Component[2].CompName = "Close out long position";
                Component[3].DataType = IndComponentType.ForceCloseShort;
                Component[3].CompName = "Close out short position";
            }

            switch (IndParam.ListParam[0].Text)
            {
                case "The Squeeze Break line rises":
                    OscillatorLogic(iFirstBar, iPrvs, adSqzBrk, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_rises);
                    break;

                case "The Squeeze Break line falls":
                    OscillatorLogic(iFirstBar, iPrvs, adSqzBrk, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_falls);
                    break;

                case "The Squeeze Break line is higher than zero":
                    OscillatorLogic(iFirstBar, iPrvs, adSqzBrk, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_is_higher_than_the_level_line);
                    break;

                case "The Squeeze Break line is lower than zero":
                    OscillatorLogic(iFirstBar, iPrvs, adSqzBrk, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_is_lower_than_the_level_line);
                    break;

                case "The Squeeze Break line crosses the zero line upward":
                    OscillatorLogic(iFirstBar, iPrvs, adSqzBrk, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_crosses_the_level_line_upward);
                    break;

                case "The Squeeze Break line crosses the zero line downward":
                    OscillatorLogic(iFirstBar, iPrvs, adSqzBrk, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_crosses_the_level_line_downward);
                    break;

                case "The Squeeze Break line changes its direction upward":
                    OscillatorLogic(iFirstBar, iPrvs, adSqzBrk, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_changes_its_direction_upward);
                    break;

                case "The Squeeze Break line changes its direction downward":
                    OscillatorLogic(iFirstBar, iPrvs, adSqzBrk, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_changes_its_direction_downward);
                    break;

                case "The Squeeze Break line crosses the Signal line upward":
                    IndicatorCrossesAnotherIndicatorUpwardLogic(iFirstBar, iPrvs, adSqzBrk, adMASignal, ref Component[2], ref Component[3]);
                    break;

                case "The Squeeze Break line crosses the Signal line downward":
                    IndicatorCrossesAnotherIndicatorDownwardLogic(iFirstBar, iPrvs, adSqzBrk, adMASignal, ref Component[2], ref Component[3]);
                    break;

                case "The Squeeze Break line is higher than the Signal line":
                    IndicatorIsHigherThanAnotherIndicatorLogic(iFirstBar, iPrvs, adSqzBrk, adMASignal, ref Component[2], ref Component[3]);
                    break;

                case "The Squeeze Break line is lower than the Signal line":
                    IndicatorIsLowerThanAnotherIndicatorLogic(iFirstBar, iPrvs, adSqzBrk, adMASignal, ref Component[2], ref Component[3]);
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            EntryFilterLongDescription  = ToString() + "; the Squeeze Break line ";
            EntryFilterShortDescription = ToString() + "; the Squeeze Break line ";
            ExitFilterLongDescription   = ToString() + "; the Squeeze Break line ";
            ExitFilterShortDescription  = ToString() + "; the Squeeze Break line ";

            switch (IndParam.ListParam[0].Text)
            {
                case "The Squeeze Break line rises":
                    EntryFilterLongDescription  += "rises";
                    EntryFilterShortDescription += "falls";
                    ExitFilterLongDescription   += "rises";
                    ExitFilterShortDescription  += "falls";
                    break;

                case "The Squeeze Break line falls":
                    EntryFilterLongDescription  += "falls";
                    EntryFilterShortDescription += "rises";
                    ExitFilterLongDescription   += "falls";
                    ExitFilterShortDescription  += "rises";
                    break;

                case "The Squeeze Break line is higher than zero":
                    EntryFilterLongDescription  += "is higher than the zero line";
                    EntryFilterShortDescription += "is lower than the zero line";
                    ExitFilterLongDescription   += "is higher than the zero line";
                    ExitFilterShortDescription  += "is lower than the zero line";
                    break;

                case "The Squeeze Break line is lower than zero":
                    EntryFilterLongDescription  += "is lower than the zero line";
                    EntryFilterShortDescription += "is higher than the zero line";
                    ExitFilterLongDescription   += "is lower than the zero line";
                    ExitFilterShortDescription  += "is higher than the zero line";
                    break;

                case "The Squeeze Break line crosses the zero line upward":
                    EntryFilterLongDescription  += "crosses the zero line upward";
                    EntryFilterShortDescription += "crosses the zero line downward";
                    ExitFilterLongDescription   += "crosses the zero line upward";
                    ExitFilterShortDescription  += "crosses the zero line downward";
                    break;

                case "The Squeeze Break line crosses the zero line downward":
                    EntryFilterLongDescription  += "crosses the zero line downward";
                    EntryFilterShortDescription += "crosses the zero line upward";
                    ExitFilterLongDescription   += "crosses the zero line downward";
                    ExitFilterShortDescription  += "crosses the zero line upward";
                    break;

                case "The Squeeze Break line changes its direction upward":
                    EntryFilterLongDescription  += "changes its direction upward";
                    EntryFilterShortDescription += "changes its direction downward";
                    ExitFilterLongDescription   += "changes its direction upward";
                    ExitFilterShortDescription  += "changes its direction downward";
                    break;

                case "The Squeeze Break line changes its direction downward":
                    EntryFilterLongDescription  += "changes its direction downward";
                    EntryFilterShortDescription += "changes its direction upward";
                    ExitFilterLongDescription   += "changes its direction downward";
                    ExitFilterShortDescription  += "changes its direction upward";
                    break;

                case "The Squeeze Break line is higher than the Signal line":
                    EntryFilterLongDescription  += "is higher than the Signal line";
                    EntryFilterShortDescription += "is lower than the Signal line";
                    ExitFilterLongDescription   += "is higher than the Signal line";
                    ExitFilterShortDescription  += "is lower than the Signal line";
                    break;

                case "The Squeeze Break line is lower than the Signal line":
                    EntryFilterLongDescription  += "is lower than the Signal line";
                    EntryFilterShortDescription += "is higher than the Signal line";
                    ExitFilterLongDescription   += "is lower than the Signal line";
                    ExitFilterShortDescription  += "is higher than the Signal line";
                    break;

                case "The Squeeze Break line crosses the Signal line upward":
                    EntryFilterLongDescription  += "crosses the Signal line upward";
                    EntryFilterShortDescription += "crosses the Signal line downward";
                    ExitFilterLongDescription   += "crosses the Signal line upward";
                    ExitFilterShortDescription  += "crosses the Signal line downward";
                    break;

                case "The Squeeze Break line crosses the Signal line downward":
                    EntryFilterLongDescription  += "crosses the Signal line downward";
                    EntryFilterShortDescription += "crosses the Signal line upward";
                    ExitFilterLongDescription   += "crosses the Signal line downward";
                    ExitFilterShortDescription  += "crosses the Signal line upward";
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName +
                (IndParam.CheckParam[0].Checked ? "* (" : " (") +
                IndParam.NumParam[0].ValueToString + ", " +  // MA period
				IndParam.NumParam[1].ValueToString + ", " +  // ATR period
				IndParam.NumParam[2].ValueToString + ", " +  // KC multiplier
				IndParam.NumParam[3].ValueToString + ")";    // BB multiplier
            return sString;
        }
    }
}
